Package: airpart
Title: Differential cell-type-specific allelic imbalance
Version: 1.19.0
Authors@R: 
    c(person(given = "Wancen",
           family = "Mu",
           role = c("aut", "cre"),
           email = "wancen@live.unc.edu",
           comment = c(ORCID = "0000-0002-5061-7581")),
    person(given = "Michael",
           family = "Love",
           role = c("aut", "ctb"),
           comment = c(ORCID = "0000-0001-8401-0545")))
Description: Airpart identifies sets of genes displaying differential
    cell-type-specific allelic imbalance across cell types
    or states, utilizing single-cell allelic counts. It makes use of a
    generalized fused lasso with binomial observations of allelic
    counts to partition cell types by their allelic
    imbalance. Alternatively, a nonparametric method for partitioning
    cell types is offered. The package includes a number of
    visualizations and quality control functions for examining single
    cell allelic imbalance datasets.
License: GPL-2
Depends: R (>= 4.1)
Imports: SingleCellExperiment, SummarizedExperiment, S4Vectors, scater,
        stats, smurf, apeglm (>= 1.13.3), emdbook, mclust, clue,
        dynamicTreeCut, matrixStats, dplyr, plyr, ggplot2,
        ComplexHeatmap, forestplot, RColorBrewer, rlang, lpSolve, grid,
        grDevices, graphics, utils, pbapply
Suggests: knitr, rmarkdown, roxygen2 (>= 6.0.0), testthat (>= 3.0.0),
        gplots, tidyr
VignetteBuilder: knitr
biocViews: SingleCell, RNASeq, ATACSeq, ChIPSeq, Sequencing,
        GeneRegulation, GeneExpression, Transcription,
        TranscriptomeVariant, CellBiology, FunctionalGenomics,
        DifferentialExpression, GraphAndNetwork, Regression,
        Clustering, QualityControl
Encoding: UTF-8
BugReports: https://github.com/Wancen/airpart/issues
URL: https://github.com/Wancen/airpart
RoxygenNote: 7.2.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/airpart
git_branch: devel
git_last_commit: b69681a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-03
NeedsCompilation: no
Packaged: 2025-11-03 21:09:22 UTC; biocbuild
Author: Wancen Mu [aut, cre] (ORCID: <https://orcid.org/0000-0002-5061-7581>),
  Michael Love [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-8401-0545>)
Maintainer: Wancen Mu <wancen@live.unc.edu>
