# airpart
Differential allelic imbalance partition across group of cells

![framwork](https://github.com/Wancen/airpartpaper/blob/main/framework.jpg)

# Install packages
```
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("airpart")
```

# Documentation
To view documentation for the version of this package installed in your system, start R and enter:
```
browseVignettes("airpart")
```

# Citation 
from within R, enter `citation("airpart")`:

Mu W, Sarkar H, Srivastava A, Choi K, Patro R, Love MI (2022). “Airpart: Interpretable statistical models for analyzing allelic imbalance in single-cell datasets.” Bioinformatics. doi: 10.1093/bioinformatics/btac212.
